@@ -1,4 +1,4 @@
-package Zonemaster::Nameserver::Cache v0.1.0;
+package Zonemaster::Nameserver::Cache v1.0.0;
 
 use 5.14.2;
 use warnings;
@@ -25,8 +25,14 @@ around 'new' => sub {
     return $object_cache{ $obj->address->ip };
 };
 
+sub empty_cache {
+    %object_cache = ();
+
+    return;
+}
+
 no Moose;
-__PACKAGE__->meta->make_immutable(inline_constructor => 0);
+__PACKAGE__->meta->make_immutable( inline_constructor => 0 );
 
 1;
 
@@ -52,4 +58,14 @@ A reference to a hash holding the cache of sent queries. Not meant for external
 
 =back
 
+=head1 CLASS METHODS
+
+=over
+
+=item empty_cache()
+
+Clear the cache.
+
+=back
+
 =cut
